/* -*-c-*-
 *
 * $Id: key.h,v 1.2 1999/12/10 23:29:48 mdw Exp $
 *
 * Simple key management
 *
 * (c) 1999 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: key.h,v $
 * Revision 1.2  1999/12/10 23:29:48  mdw
 * Change header file guard names.
 *
 * Revision 1.1  1999/09/03 08:41:12  mdw
 * Initial import.
 *
 */

#ifndef CATACOMB_KEY_H
#define CATACOMB_KEY_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>
#include <time.h>

#include <mLib/bits.h>
#include <mLib/hash.h>
#include <mLib/sym.h>

/*----- Data structures ---------------------------------------------------*/

/* --- Key attributes --- *
 *
 * Each attribute is stored as a symbol in a symbol table.  The value is
 * the plain (not url-encoded) text to be written to the the file.  If the
 * value is binary data, then by this point it's base-64 encoded.
 */

typedef struct key_attr {
  sym_base _b;				/* Symbol table data */
  char *p;				/* Pointer to attribute value */
} key_attr;

/* --- Main key structure --- *
 *
 * Each key is stored in two symbol tables, one indexed by keyid, and the
 * other indexed by type.  Because many keys can have the same type, the type
 * table contains a list of keys, sorted in descending order of expiry.
 */

typedef struct key {
  hash_base _b;				/* Symbol table data */
  struct key *next;			/* Next key of the same type */
  uint32 id;				/* Key id used to name it */
  char *type;				/* Textual key type */
  void *k;				/* Actual key data */
  size_t ksz;				/* Size of the key data */
  time_t exp, del;			/* Expiry times for keys */
  sym_table a;				/* Hashtable of key attributes */
  char *c;				/* Any additional comments */
} key;

/* --- The keys-by-type entries --- */

typedef struct key_type {
  sym_base _b;				/* Symbol table data */
  key *k;				/* Pointer to first key in list */
} key_type;

/* --- A key file --- */

typedef struct key_file {
  FILE *fp;				/* File pointer open on file */
  int fd;				/* File descriptor open on file */
  char *name;				/* Filename used to create it */
  unsigned f;				/* Various useful flags */
  hash_table byid;			/* Table of keys by keyid */
  sym_table bytype;			/* Table of keys by type */
  size_t idload;			/* Loading on id table */
} key_file;

/* --- Key file flags --- */

enum {
  KF_WRITE = 1,				/* File opened for writing */
  KF_MODIFIED = 2			/* File has been modified */
};

/* --- Iterating over keys --- *
 *
 * Both of these are simple symbol table iterators, but they're made distinct
 * types for the dubious benefits that type safety brings.
 */

typedef struct { hash_iter i; time_t t; } key_iter;
typedef struct { sym_iter i; } key_attriter;

/* --- File opening options --- */

enum {
  KOPEN_READ,
  KOPEN_WRITE
};

/* --- Various other magic numbers --- */

#define KEXP_UNUSED ((time_t)0)		/* Key has never been used */
#define KEXP_FOREVER ((time_t)-1)	/* Never expire this key */
#define KEXP_EXPIRE ((time_t)-2)	/* Expire this key when unused */

/* --- Write attempt codes --- */

enum {
  KWRITE_OK,				/* Everything went fine */
  KWRITE_FAIL = -1,			/* Close attempt failed */
  KWRITE_BROKEN	= -2			/* Key ring needs manual fixing */
};

/* --- Macros for testing expiry --- */

#define KEY_EXPIRED(now, exp)						\
  ((exp) == KEXP_EXPIRE || ((exp) != KEXP_FOREVER && (exp) < (now)))

#define KEY_DELETED(now, del) ((del) == KEXP_FOREVER || (del) < (now))

/*----- Functions provided ------------------------------------------------*/

/* --- @key_chktype@ --- *
 *
 * Arguments:	@const char *type@ = pointer to a type string
 *
 * Returns:	Zero if OK, -1 on error.
 *
 * Use:		Checks whether a type string is OK.
 */

extern int key_chktype(const char */*type*/);

/* --- @key_chkcomment@ --- *
 *
 * Arguments:	@const char *comment@ = pointer to a comment string
 *
 * Returns:	Zero if OK, -1 on error.
 *
 * Use:		Checks whether a comment string is OK.
 */

extern int key_chkcomment(const char */*c*/);

/* --- @key_mkiter@ --- *
 *
 * Arguments:	@key_iter *i@ = pointer to iterator object
 *		@key_file *f@ = pointer to file structure
 *
 * Returns:	---
 *
 * Use:		Initializes a key iterator.  The keys are returned by
 *		@key_next@.
 */

extern void key_mkiter(key_iter */*i*/, key_file */*f*/);

/* --- @key_next@ --- *
 *
 * Arguments:	@key_iter *i@ = pointer to iterator object
 *
 * Returns:	Pointer to next key, or null.
 *
 * Use:		Returns the next key in some arbitrary sequence.
 */

extern key *key_next(key_iter */*i*/);

/* --- @key_mkattriter@ --- *
 *
 * Arguments:	@key_attriter *i@ = pointer to attribute iterator
 *		@key_file *f@ = pointer to key file
 *		@key *k@ = pointer to key
 *
 * Returns:	---
 *
 * Use:		Initializes an attribute iterator.  The attributes are
 *		returned by @key_nextattr@.
 */

extern void key_mkattriter(key_attriter */*i*/, key_file */*f*/, key */*k*/);

/* --- @key_nextattr@ --- *
 *
 * Arguments:	@key_attriter *i@ = pointer to attribute iterator
 *		@const char **n, **v@ = pointers to name and value
 *
 * Returns:	Zero if no attribute available, or nonzero if returned OK.
 *
 * Use:		Returns the next attribute.
 */

extern int key_nextattr(key_attriter */*i*/,
			const char **/*n*/, const char **/*v*/);

/* --- @key_bytype@ --- *
 *
 * Arguments:	@key_file *f@ = key file we want a key from
 *		@const char *type@ = type string for desired key
 *
 * Returns:	Pointer to the best key to use, or null.
 *
 * Use:		Looks up a key by its type.  Returns the key with the latest
 *		expiry time.  This function will not return an expired key.
 */

extern key *key_bytype(key_file */*f*/, const char */*type*/);

/* --- @key_byid@ --- *
 *
 * Arguments:	@key_file *f@ = key file to find a key from
 *		@uint32 id@ = id to look for
 *
 * Returns:	Key with matching id.
 *
 * Use:		Returns a key given its id.  This function will return an
 *		expired key, but not a deleted one.
 */

extern key *key_byid(key_file */*f*/, uint32 /*id*/);

/* --- @key_getattr@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file
 *		@key *k@ = pointer to key
 *		@const char *n@ = pointer to attribute name
 *
 * Returns:	Pointer to attribute value, or null if not found.
 *
 * Use:		Returns the value of a key attribute.
 */

extern const char *key_getattr(key_file */*f*/, key */*k*/,
			       const char */*n*/);

/* --- @key_putattr@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file
 *		@key *k@ = pointer to key
 *		@const char *n@ = pointer to attribute name
 *		@const char *v@ = pointer to attribute value
 *
 * Returns:	---
 *
 * Use:		Inserts an attribute on a key.  If an attribute with the same
 *		name already exists, it is deleted.
 */

extern void key_putattr(key_file */*f*/, key */*k*/,
			const char */*n*/, const char */*v*/);

/* --- @key_setcomment@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to key file block
 *		@key *k@ = pointer to key block
 *		@const char *c@ = pointer to comment to set, or zero
 *
 * Returns:	---
 *
 * Use:		Replaces the key's current comment with a new one.
 */

extern void key_setcomment(key_file */*f*/, key */*k*/, const char */*c*/);

/* --- @key_merge@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file structure
 *		@const char *file@ = name of file (for error messages)
 *		@FILE *fp@ = file handle to read from
 *
 * Returns:	---
 *
 * Use:		Reads keys from a file, and inserts them into the file.
 */

extern void key_merge(key_file */*f*/, const char */*file*/, FILE */*fp*/);

/* --- @key_extract@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file structure
 *		@key *k@ = key to extract
 *		@FILE *fp@ = file to write on
 *
 * Returns:	Zero if OK, EOF on error.
 *
 * Use:		Extracts a key to an ouptut file.
 */

extern int key_extract(key_file */*f*/, key */*k*/, FILE */*fp*/);

/* --- @key_write@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to key file block
 *
 * Returns:	A @KWRITE_@ code indicating how well it worked.
 *
 * Use:		Writes a key file's data back to the actual file.  This code
 *		is extremely careful about error handling.  It should usually
 *		be able to back out somewhere sensible, but it can tell when
 *		it's got itself into a real pickle and starts leaving well
 *		alone.
 *
 *		Callers, please make sure that you ring alarm bells when this
 *		function returns @KWRITE_BROKEN@.
 */

extern int key_write(key_file */*f*/);

/* --- @key_open@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file structure to initialize
 *		@const char *file@ = pointer to the file name
 *		@int how@ = opening options (@KOPEN_*@).
 *
 * Returns:	Zero if it worked, nonzero otherwise.
 *
 * Use:		Opens a key file, reads its contents, and stores them in a
 *		structure.  The file is locked appropriately until closed
 *		using @key_close@.  On an error, everything is cleared away
 *		tidily.  If the file is opened with @KOPEN_WRITE@, it's
 *		created if necessary, with read and write permissions for its
 *		owner only.
 */

extern int key_open(key_file */*f*/, const char */*file*/, int /*how*/);

/* --- @key_close@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to key file block
 *
 * Returns:	A @KWRITE_@ code indicating how it went.
 *
 * Use:		Frees all the key data, writes any changes.  Make sure that
 *		all hell breaks loose if this returns @KWRITE_BROKEN@.
 */

extern int key_close(key_file */*f*/);

/* --- @key_new@ ---
 *
 * Arguments:	@key_file *f@ = pointer to key file
 *		@const char *type@ = the type of this key
 *		@const void *k@ = pointer to key data
 *		@size_t ksz@ = size of key data
 *		@time_t exp@ = when the key expires
 *		@const char *c@ = textual comment to attach
 *
 * Returns:	Key block containing new data, or null if it couldn't be
 *		done.
 *
 * Use:		Attaches a new key to a key file.  You must have a writable
 *		key file for this to work.
 *
 *		The type is a key type string.  This interface doesn't care
 *		about how type strings are formatted: it just treats them as
 *		opaque gobs of text.  Clients are advised to choose some
 *		standard for representing key types, though.
 *
 *		The key can be any old binary mess.
 *
 *		The expiry time should either be a time in the future, or the
 *		magic value @KEXP_FOREVER@ which means `never expire this
 *		key'.  Be careful with `forever' keys.  If I were you, I'd
 *		use a more sophisticated key management system than this for
 *		them.
 *
 *		The comment can be any old text not containing newlines or
 *		nulls.  This interface doesn't impose any length restrictions
 *		on comment lengths.
 */

extern key *key_new(key_file */*f*/, const char */*type*/,
		    const void */*k*/, size_t /*ksz*/,
		    time_t /*exp*/, const char */*c*/);

/* --- @key_delete@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file block
 *		@key *k@ = key to delete
 *
 * Returns:	---
 *
 * Use:		Removes the given key from the list.  The key file must be
 *		writable.  (Due to the horridness of the data structures,
 *		deleted keys aren't actually removed, just marked so that
 *		they can't be looked up or iterated over.  One upshot of
 *		this is that they don't get written back to the file when
 *		it's closed.)
 */

extern void key_delete(key_file */*f*/, key */*k*/);

/* --- @key_expire@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to file block
 *		@key *k@ = pointer to key block
 *
 * Returns:	---
 *
 * Use:		Immediately marks the key as expired.  It may be removed
 *		immediately, if it is no longer required, and will be removed
 *		by a tidy operation when it is no longer required.  The key
 *		file must be writable.
 */

extern void key_expire(key_file */*f*/, key */*k*/);

/* --- @key_used@ --- *
 *
 * Arguments:	@key_file *f@ = pointer to key file
 *		@key *k@ = pointer to key block
 *		@time_t t@ = when key can be removed
 *
 * Returns:	Zero if OK, nonzero on failure.
 *
 * Use:		Marks a key as being required until a given time.  Even
 *		though the key may expire before then (and won't be returned
 *		by type after that time), it will still be available when
 *		requested explicitly by id.  The key file must be writable.
 *
 *		The only (current) reason for failure is attempting to use
 *		a key which can expire for something which can't.
 */

extern int key_used(key_file */*f*/, key */*k*/, time_t /*t*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
